/*
 * Redberry: symbolic tensor computations.
 *
 * Copyright (c) 2010-2013:
 *   Stanislav Poslavsky   <stvlpos@mail.ru>
 *   Bolotin Dmitriy       <bolotin.dmitriy@gmail.com>
 *
 * This file is part of Redberry.
 *
 * Redberry is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Redberry is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Redberry. If not, see <http://www.gnu.org/licenses/>.
 */
package cc.redberry.physics.feyncalc;

import cc.redberry.core.indices.IndexType;
import cc.redberry.core.indices.IndicesUtils;
import cc.redberry.core.tensor.SimpleTensor;
import cc.redberry.core.tensor.Tensor;

/**
 * @author Dmitry Bolotin
 * @author Stanislav Poslavsky
 */
public final class MatrixUtils {
    private MatrixUtils() {
    }

    public static boolean isMatrixOrVectorOrCovector(Tensor t, IndexType matrixType) {
        int size;
        return t instanceof SimpleTensor
                && ((size = t.getIndices().size(matrixType)) == 2
                || size == 1);
    }

    public static boolean isVectorOrCovector(Tensor t, IndexType matrixType) {
        return t instanceof SimpleTensor
                && t.getIndices().size(matrixType) == 1;
    }

    public static boolean isMatrix(Tensor t, IndexType matrixType) {
        return t instanceof SimpleTensor
                && t.getIndices().size(matrixType) == 2;
    }

    public static boolean isVector(Tensor t, IndexType matrixType) {
        return t instanceof SimpleTensor
                && t.getIndices().size(matrixType) == 1
                && IndicesUtils.getState(t.getIndices().get(matrixType, 0));
    }

    public static boolean isCovector(Tensor t, IndexType matrixType) {
        return t instanceof SimpleTensor
                && t.getIndices().size(matrixType) == 1
                && !IndicesUtils.getState(t.getIndices().get(matrixType, 0));
    }
}
